/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import React, { useState, useRef, useEffect } from 'react';
import { useAuthCache, useAuthentication } from '@graphpolaris/shared/lib/data-access';
import { DropdownItem } from '@graphpolaris/shared/lib/components/dropdowns';
import GpLogo from './gp-logo';
import { Popover, PopoverContent, PopoverTrigger } from '@graphpolaris/shared/lib/components/layout/Popover';
import { FeatureEnabled } from '@graphpolaris/shared/lib/components/featureFlags';
import { Button, useActiveSaveStateAuthorization } from '@graphpolaris/shared';
import { ManagementTrigger, ManagementViews } from '@graphpolaris/shared/lib/management';
import { getEnvVariable } from 'config';

export const Navbar = () => {
  const dropdownRef = useRef<HTMLDivElement>(null);
  const auth = useAuthentication();
  const authCache = useAuthCache();
  const authorization = useActiveSaveStateAuthorization();
  const [menuOpen, setMenuOpen] = useState(false);
  const buildInfo = getEnvVariable('GRAPHPOLARIS_VERSION');
  const [managementOpen, setManagementOpen] = useState<boolean>(false);
  const [current, setCurrent] = useState<ManagementViews>('overview');

  useEffect(() => {
    const handleClickOutside = (event: MouseEvent) => {
      if (dropdownRef.current && !dropdownRef.current.contains(event.target as Node)) {
        setMenuOpen(false);
      }
    };
    if (menuOpen) document.addEventListener('mousedown', handleClickOutside);
    return () => {
      document.removeEventListener('mousedown', handleClickOutside);
    };
  }, [menuOpen]);

  return (
    <nav className="w-full px-2 h-12 flex flex-row items-center gap-2 md:gap-3 lg:gap-4">
      <GpLogo className="h-7" includeText={false} />
      <ManagementTrigger managementOpen={managementOpen} setManagementOpen={setManagementOpen} current={current} setCurrent={setCurrent} />

      <Button label="Share" variantType="primary" size="sm" onClick={() => auth.newShareRoom()} />
      <div className="ml-auto">
        <div className="w-fit" ref={dropdownRef}>
          <Popover>
            <PopoverTrigger>
              <div
                className="relative inline-flex items-center justify-center w-8 h-8 overflow-hidden bg-secondary-500 rounded-full hover:bg-secondary-600 transition-colors duration-150 ease-in-out cursor-pointer"
                onClick={() => setMenuOpen(!menuOpen)}
              >
                <span className="font-medium text-light">{authCache.authentication?.username?.slice(0, 2).toUpperCase()}</span>
              </div>
            </PopoverTrigger>

            <PopoverContent className="w-56 z-30 bg-light rounded-sm border-[1px] outline-none">
              <div className="p-2 text-sm border-b">
                <h2 className="font-bold">user: {authCache.authentication?.username}</h2>
                <h3 className="text-xs break-words">session: {authCache.authentication?.sessionID}</h3>
                <h3 className="text-xs break-words">license: Creator</h3>
              </div>
              {authCache.authentication?.authenticated ? (
                <>
                  <FeatureEnabled featureFlag="SHARABLE_EXPLORATION">
                    <DropdownItem
                      value="Share"
                      onClick={() => {
                        auth.newShareRoom();
                      }}
                    />
                  </FeatureEnabled>
                  <FeatureEnabled featureFlag="VIEWER_PERMISSIONS">
                    {authCache.authorization?.savestate?.W && authorization.database?.W && (
                      <DropdownItem
                        value="Viewer Permissions"
                        onClick={() => {
                          setManagementOpen(true);
                          setCurrent('members');
                        }}
                      />
                    )}
                  </FeatureEnabled>
                  <DropdownItem value="Settings" onClick={() => {}} />
                  <DropdownItem
                    value="Log out"
                    onClick={() => {
                      location.replace(`${getEnvVariable('GP_AUTH_URL')}/outpost.goauthentik.io/sign_out`);
                    }}
                  />
                </>
              ) : (
                <>
                  <DropdownItem value="Login" onClick={() => {}} />
                </>
              )}

              {authCache.authentication?.roomID && (
                <div className="p-2 border-b">
                  <h3 className="text-xs break-words">Share ID: {authCache.authentication?.roomID}</h3>
                </div>
              )}
              <div className="p-2 border-t">
                <h3 className="text-xs">Version: {buildInfo}</h3>
              </div>
            </PopoverContent>
          </Popover>
        </div>
      </div>
    </nav>
  );
};
